@@ -21,13 +21,20 @@
 
 #include <glib-object.h>
 #include <babl/babl.h>
+#include <gegl-matrix.h>
+#include <gegl-enums.h>
 
 G_BEGIN_DECLS
 
 #define GEGL_TYPE_BUFFER (gegl_buffer_get_type ())
 #define GEGL_BUFFER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEGL_TYPE_BUFFER, GeglBuffer))
+#define GEGL_IS_BUFFER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEGL_TYPE_BUFFER))
 #ifndef __GEGL_BUFFER_TYPES_H__
-typedef struct _GeglBuffer     GeglBuffer;
+typedef struct _GeglBuffer   GeglBuffer;
+typedef struct _GeglSampler  GeglSampler;
+#endif
+#ifndef __GEGL_BUFFER_BACKEND_H__
+typedef struct _GeglTileBackend GeglTileBackend;
 #endif
 
 /***
@@ -44,7 +51,7 @@ typedef struct _GeglBuffer     GeglBuffer;
 GType           gegl_buffer_get_type          (void) G_GNUC_CONST;
 
 /**
- * gegl_buffer_new:
+ * gegl_buffer_new: (skip)
  * @extent: the geometry of the buffer (origin, width and height) a
  * GeglRectangle.
  * @format: the Babl pixel format to be used, create one with babl_format("RGBA
@@ -54,24 +61,61 @@ GType           gegl_buffer_get_type          (void) G_GNUC_CONST;
  * possible to pass in NULL for both extent and format, a NULL extent creates
  * an empty buffer and a NULL format makes the buffer default to "RGBA float".
  */
-GeglBuffer*     gegl_buffer_new               (const GeglRectangle *extent,
+GeglBuffer *    gegl_buffer_new               (const GeglRectangle *extent,
                                                const Babl          *format);
 
-
+/**
+ * gegl_buffer_introspectable_new:
+ * @format_name: The Babl format name for this buffer, e.g. "RGBA float"
+ * @x: x origin of the buffer's extent
+ * @y: y origin of the buffer's extent
+ * @width: width of the buffer's extent
+ * @height: height of the buffer's extent
+ *
+ * Create a new GeglBuffer with the given format and dimensions.
+ *
+ * Rename to: gegl_buffer_new
+ */
+GeglBuffer *
+gegl_buffer_introspectable_new (const char *format_name,
+                                gint x,
+                                gint y,
+                                gint width,
+                                gint height);
 
 /**
  * gegl_buffer_new_for_backend:
  * @extent: the geometry of the buffer (origin, width and height) a
  * GeglRectangle.
+ * @backend: an instance of a GeglTileBackend subclass.
  *
  * Create a new GeglBuffer from a backend, if NULL is passed in the extent of
  * the buffer will be inherited from the extent of the backend.
  *
  * returns a GeglBuffer, that holds a reference to the provided backend.
  */
-GeglBuffer *
-gegl_buffer_new_for_backend (const GeglRectangle *extent,
-                             void                *backend);
+GeglBuffer *   gegl_buffer_new_for_backend    (const GeglRectangle *extent,
+                                               GeglTileBackend     *backend);
+
+/**
+ * gegl_buffer_add_handler:
+ * @buffer: a #GeglBuffer
+ * @handler: a #GeglTileHandler
+ *
+ * Add a new tile handler in the existing chain of tile handler of a GeglBuffer.
+ */
+void           gegl_buffer_add_handler        (GeglBuffer          *buffer,
+                                               gpointer             handler);
+
+/**
+ * gegl_buffer_remove_handler:
+ * @buffer: a #GeglBuffer
+ * @handler: a #GeglTileHandler
+ *
+ * Remove the provided tile handler in the existing chain of tile handler of a GeglBuffer.
+ */
+void           gegl_buffer_remove_handler     (GeglBuffer          *buffer,
+                                               gpointer             handler);
 
 /**
  * gegl_buffer_open:
@@ -81,13 +125,13 @@ gegl_buffer_new_for_backend (const GeglRectangle *extent,
  * state so multiple instances of gegl can share the same buffer. Sets on
  * one buffer are reflected in the other.
  *
- * Returns: a GeglBuffer object.
+ * Returns: (transfer full): a GeglBuffer object.
  */
-GeglBuffer*     gegl_buffer_open              (const gchar         *path);
+GeglBuffer *    gegl_buffer_open              (const gchar         *path);
 
 /**
  * gegl_buffer_save:
- * @buffer: a #GeglBuffer.
+ * @buffer: (transfer none): a #GeglBuffer.
  * @path: the path where the gegl buffer will be saved, any writable GIO uri is valid.
  * @roi: the region of interest to write, this is the tiles that will be collected and
  * written to disk.
@@ -106,9 +150,9 @@ void            gegl_buffer_save              (GeglBuffer          *buffer,
  * gegl_buffer_save it should be possible to open through any GIO transport, buffers
  * that have been used as swap needs random access to be opened.
  *
- * Returns: a #GeglBuffer object.
+ * Returns: (transfer full): a #GeglBuffer object.
  */
-GeglBuffer     *gegl_buffer_load              (const gchar         *path);
+GeglBuffer *     gegl_buffer_load             (const gchar         *path);
 
 /**
  * gegl_buffer_flush:
@@ -122,23 +166,15 @@ void            gegl_buffer_flush             (GeglBuffer          *buffer);
 
 /**
  * gegl_buffer_create_sub_buffer:
- * @buffer: parent buffer.
- * @extent: coordinates of new buffer.
+ * @buffer: (transfer none): parent buffer.
+ * @extent: (transfer none): coordinates of new buffer.
  *
  * Create a new sub GeglBuffer, that is a view on a larger buffer.
- */
-GeglBuffer*     gegl_buffer_create_sub_buffer (GeglBuffer          *buffer,
-                                               const GeglRectangle *extent);
-
-/**
- * gegl_buffer_destroy:
- * @buffer: the buffer we're done with.
  *
- * Destroys a buffer and frees up the resources used by a buffer, internally
- * this is done with reference counting and gobject, and gegl_buffer_destroy
- * is a thin wrapper around g_object_unref.
- **/
-void            gegl_buffer_destroy           (GeglBuffer          *buffer);
+ * Return value: (transfer full): the new sub buffer
+ */
+GeglBuffer *    gegl_buffer_create_sub_buffer (GeglBuffer          *buffer,
+                                               const GeglRectangle *extent);
 
 /**
  * gegl_buffer_get_extent:
@@ -148,7 +184,7 @@ void            gegl_buffer_destroy           (GeglBuffer          *buffer);
  * specific GeglBuffer, this is also the default width/height of buffers passed
  * in to gegl_buffer_set and gegl_buffer_get (with a scale of 1.0 at least).
  */
-const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
+const GeglRectangle * gegl_buffer_get_extent  (GeglBuffer *buffer);
 
 
 /**
@@ -162,8 +198,20 @@ const GeglRectangle * gegl_buffer_get_extent (GeglBuffer *buffer);
  *
  * Returns TRUE if the change of extent was succesful.
  */
-gboolean gegl_buffer_set_extent (GeglBuffer          *buffer,
-                                 const GeglRectangle *extent);
+gboolean          gegl_buffer_set_extent      (GeglBuffer          *buffer,
+                                               const GeglRectangle *extent);
+
+/**
+ * gegl_buffer_set_abyss:
+ * @buffer: the buffer to operate on.
+ * @abyss: new abyss.
+ *
+ * Changes the size and position of the abyss rectangle of a buffer.
+ *
+ * Returns TRUE if the change of abyss was succesful.
+ */
+gboolean          gegl_buffer_set_abyss      (GeglBuffer          *buffer,
+                                              const GeglRectangle *abyss);
 
 /* convenience access macros */
 
@@ -212,17 +260,22 @@ gboolean gegl_buffer_set_extent (GeglBuffer          *buffer,
 #endif
 
 /**
- * gegl_buffer_get:
+ * gegl_buffer_get: (skip)
  * @buffer: the buffer to retrieve data from.
- * @scale: sampling scale, 1.0 = pixel for pixel 2.0 = magnify, 0.5 scale down.
  * @rect: the coordinates we want to retrieve data from, and width/height of
  * destination buffer, if NULL equal to the extent of the buffer. The
  * coordinates and dimensions are after scale has been applied.
+ * @scale: sampling scale, 1.0 = pixel for pixel 2.0 = magnify, 0.5 scale down.
  * @format: the BablFormat to store in the linear buffer @dest.
  * @dest: the memory destination for a linear buffer for the pixels, the size needed
  * depends on the requested BablFormat.
  * @rowstride: rowstride in bytes, or GEGL_AUTO_ROWSTRIDE to compute the
  * rowstride based on the width and bytes per pixel for the specified format.
+ * @repeat_mode: how requests outside the buffer extent are handled.
+ * Valid values: GEGL_ABYSS_NONE (abyss pixels are zeroed), GEGL_ABYSS_WHITE
+ * (abyss pixels are white), GEGL_ABYSS_BLACK (abyss pixels are black),
+ * GEGL_ABYSS_CLAMP (coordinates are clamped to the abyss rectangle),
+ * GEGL_ABYSS_LOOP (buffer contents are tiled if outside of the abyss rectangle).
  *
  * Fetch a rectangular linear buffer of pixel data from the GeglBuffer, the
  * data is converted to the desired BablFormat, if the BablFormat stored and
@@ -230,16 +283,21 @@ gboolean gegl_buffer_set_extent (GeglBuffer          *buffer,
  * the tile structure into a linear buffer.
  */
 void            gegl_buffer_get               (GeglBuffer          *buffer,
-                                               gdouble              scale,
                                                const GeglRectangle *rect,
+                                               gdouble              scale,
                                                const Babl          *format,
                                                gpointer             dest,
-                                               gint                 rowstride);
+                                               gint                 rowstride,
+                                               GeglAbyssPolicy      repeat_mode);
 
 /**
  * gegl_buffer_set:
  * @buffer: the buffer to modify.
- * @rect: the coordinates we want to change the data of and the width/height extent, if NULL equal to the extent of the buffer.
+ * @rect: the coordinates we want to change the data of and the width/height of
+ * the linear buffer being set, scale specifies the scaling factor applied to
+ * the data when setting.
+ * @scale_level: the scale level being set, 0 = 1:1 = default = base mipmap level,
+ * 1 = 1:2, 2=1:4, 3=1:8 ..
  * @format: the babl_format the linear buffer @src.
  * @src: linear buffer of image data to be stored in @buffer.
  * @rowstride: rowstride in bytes, or GEGL_AUTO_ROWSTRIDE to compute the
@@ -249,21 +307,71 @@ void            gegl_buffer_get               (GeglBuffer          *buffer,
  */
 void            gegl_buffer_set               (GeglBuffer          *buffer,
                                                const GeglRectangle *rect,
+                                               gint                 scale_level,
                                                const Babl          *format,
-                                               void                *src,
+                                               const void          *src,
                                                gint                 rowstride);
 
 
+/**
+ * gegl_buffer_set_color:
+ * @buffer: a #GeglBuffer
+ * @rect: a rectangular region to fill with a color.
+ * @color: the GeglColor to fill with.
+ *
+ * Sets the region covered by rect to the specified color.
+ */
+void            gegl_buffer_set_color         (GeglBuffer          *buffer,
+                                               const GeglRectangle *rect,
+                                               GeglColor           *color);
+
+
+/**
+ * gegl_buffer_set_pattern:
+ * @buffer: a #GeglBuffer
+ * @rect: a rectangular region
+ * @pattern: a #GeglBuffer to be repeated as a pattern
+ * @x_offset: where the pattern starts horizontally
+ * @y_offset: where the pattern starts vertical
+ *
+ * Fill a region with a repeating pattern.
+ */
+void            gegl_buffer_set_pattern       (GeglBuffer          *buffer,
+                                               const GeglRectangle *rect,
+                                               GeglBuffer          *pattern,
+                                               gdouble              x_offset,
+                                               gdouble              y_offset);
+
 /**
  * gegl_buffer_get_format:
  * @buffer: a #GeglBuffer
  *
- * Get the native babl format of the buffer.
+ * Get the babl format of the buffer, this might not be the format the buffer
+ * was originally created with, you need to use gegl_buffer_set_format (buf,
+ * NULL); to retireve the original format (potentially having save away the
+ * original format of the buffer to re-set it.)
  *
  * Returns: the babl format used for storing pixels in the buffer.
- * 
+ *
  */
-const Babl    *gegl_buffer_get_format        (GeglBuffer           *buffer);
+const Babl *    gegl_buffer_get_format        (GeglBuffer           *buffer);
+
+
+/**
+ * gegl_buffer_set_format: (skip)
+ * @buffer: a #GeglBuffer
+ * @format: the new babl format, must have same bpp as original format.
+ *
+ * Set the babl format of the buffer, setting the babl format of the buffer
+ * requires the new format to have exactly the same bytes per pixel as the
+ * original format. If NULL is passed in the format of the buffer is reset to
+ * the original format.
+ *
+ * Returns: the new babl format or NULL if it the passed in buffer was
+ * incompatible (then the original format is still used).
+ */
+const Babl *    gegl_buffer_set_format        (GeglBuffer          *buffer,
+                                               const Babl          *format);
 
 /**
  * gegl_buffer_clear:
@@ -279,17 +387,16 @@ void            gegl_buffer_clear             (GeglBuffer          *buffer,
 
 /**
  * gegl_buffer_copy:
- * @src: source buffer.
+ * @src: (transfer none): source buffer.
  * @src_rect: source rectangle (or NULL to copy entire source buffer)
- * @dst: destination buffer.
+ * @dst: (transfer none): destination buffer.
  * @dst_rect: position of upper left destination pixel, or NULL for top
  * left coordinates of the buffer extents.
  *
  * copies a region from source buffer to destination buffer.
  *
  * If the babl_formats of the buffers are the same, and the tile boundaries
- * align, this should optimally lead to shared tiles that are copy on write,
- * this functionality is not implemented yet.
+ * align, this will create copy-on-write tiles in the destination buffer.
  */
 void            gegl_buffer_copy              (GeglBuffer          *src,
                                                const GeglRectangle *src_rect,
@@ -299,52 +406,48 @@ void            gegl_buffer_copy              (GeglBuffer          *src,
 
 /**
  * gegl_buffer_dup:
- * @buffer: the GeglBuffer to duplicate.
+ * @buffer: (transfer none): the GeglBuffer to duplicate.
  *
- * duplicate a buffer (internally uses gegl_buffer_copy), this should ideally
- * lead to a buffer that shares the raster data with the original on a tile
- * by tile COW basis. This is not yet implemented
+ * Duplicate a buffer (internally uses gegl_buffer_copy). Aligned tiles
+ * will create copy-on-write clones in the new buffer.
+ *
+ * Return value: (transfer full): the new buffer
  */
-GeglBuffer    * gegl_buffer_dup               (GeglBuffer       *buffer);
+GeglBuffer *    gegl_buffer_dup               (GeglBuffer       *buffer);
 
-typedef enum {
-  GEGL_INTERPOLATION_NEAREST = 0,
-  GEGL_INTERPOLATION_LINEAR,
-  GEGL_INTERPOLATION_CUBIC,
-  GEGL_INTERPOLATION_LANCZOS,
-  GEGL_INTERPOLATION_DOWNSHARP,
-  GEGL_INTERPOLATION_DOWNSIZE,
-  GEGL_INTERPOLATION_DOWNSMOOTH,
-  GEGL_INTERPOLATION_UPSHARP,
-  GEGL_INTERPOLATION_UPSIZE,
-  GEGL_INTERPOLATION_UPSMOOTH
-} GeglInterpolation;
 
 /**
- * gegl_buffer_sample:
+ * gegl_buffer_sample: (skip)
  * @buffer: the GeglBuffer to sample from
  * @x: x coordinate to sample in buffer coordinates
  * @y: y coordinate to sample in buffer coordinates
- * @scale: the scale we're fetching at (<1.0 can lead to decimation)
+ * @scale: a matrix that indicates scaling factors, see
+ * gegl_sampler_compute_scale the same.
  * @dest: buffer capable of storing one pixel in @format.
  * @format: the format to store the sampled color in.
- * @interpolation: the interpolation behavior to use, currently only nearest
- * neighbour is implemented for this API, bilinear, bicubic and lanczos needs
- * to be ported from working code. Valid values: GEGL_INTERPOLATION_NEAREST and
- * GEGL_INTERPOLATION_LINEAR, GEGL_INTERPOLATON_CUBIC and
- * GEGL_INTERPOLATION_LANCZOS.
+ * @sampler_type: the sampler type to use,
+ * to be ported from working code. Valid values: GEGL_SAMPLER_NEAREST,
+ * GEGL_SAMPLER_LINEAR, GEGL_SAMPLER_CUBIC, GEGL_SAMPLER_NOHALO and
+ * GEGL_SAMPLER_LOHALO
+ * @repeat_mode: how requests outside the buffer extent are handled.
+ * Valid values: GEGL_ABYSS_NONE (abyss pixels are zeroed), GEGL_ABYSS_WHITE
+ * (abyss pixels are white), GEGL_ABYSS_BLACK (abyss pixels are black),
+ * GEGL_ABYSS_CLAMP (coordinates are clamped to the abyss rectangle),
+ * GEGL_ABYSS_LOOP (buffer contents are tiled if outside of the abyss rectangle).
  *
  * Query interpolate pixel values at a given coordinate using a specified form
  * of interpolation. The samplers used cache for a small neighbourhood of the
  * buffer for more efficient access.
  */
-void            gegl_buffer_sample            (GeglBuffer       *buffer,
+void              gegl_buffer_sample          (GeglBuffer       *buffer,
                                                gdouble           x,
                                                gdouble           y,
-                                               gdouble           scale,
+                                               GeglMatrix2      *scale,
                                                gpointer          dest,
                                                const Babl       *format,
-                                               GeglInterpolation interpolation);
+                                               GeglSamplerType   sampler_type,
+                                               GeglAbyssPolicy   repeat_mode);
+
 
 
 /**
@@ -358,19 +461,68 @@ void            gegl_buffer_sample            (GeglBuffer       *buffer,
  */
 void            gegl_buffer_sample_cleanup    (GeglBuffer *buffer);
 
+
+
 /**
- * gegl_buffer_interpolation_from_string:
+ * gegl_sampler_type_from_string:
  * @string: the string to look up
  *
  * Looks up the GeglInterpolation corresponding to a string, if no matching
- * interpolation is found returns GEGL_INTERPOLATION_NEAREST.
+ * interpolation is found GEGL_SAMPLER_NEAREST is returned.
  */
-GeglInterpolation gegl_buffer_interpolation_from_string (const gchar *string);
-
-
+GeglSamplerType gegl_sampler_type_from_string (const gchar *string);
 
 /**
- * gegl_buffer_linear_new:
+ * gegl_buffer_sampler_new: (skip)
+ * @buffer: buffer to create a new sampler for
+ * @format: format we want data back in
+ * @sampler_type: the sampler type to use,
+ * to be ported from working code. Valid values: GEGL_SAMPLER_NEAREST,
+ * GEGL_SAMPLER_LINEAR, GEGL_SAMPLER_CUBIC, GEGL_SAMPLER_NOHALO and
+ * GEGL_SAMPLER_LOHALO
+ *
+ * Create a new sampler, when you are done with the sampler, g_object_unref
+ * it.
+ *
+ * Samplers only hold weak references to buffers, so if its buffer is freed
+ * the sampler will become invalid.
+ */
+GeglSampler *    gegl_buffer_sampler_new      (GeglBuffer       *buffer,
+                                               const Babl       *format,
+                                               GeglSamplerType   sampler_type);
+
+/**
+ * gegl_sampler_get:
+ * @sampler: a GeglSampler gotten from gegl_buffer_sampler_new
+ * @x: x coordinate to sample
+ * @y: y coordinate to sample
+ * @scale: matrix representing extent of sampling area in source buffer.
+ * @output: memory location for output data.
+ * @repeat_mode: how requests outside the buffer extent are handled.
+ * Valid values: GEGL_ABYSS_NONE (abyss pixels are zeroed), GEGL_ABYSS_WHITE
+ * (abyss pixels are white), GEGL_ABYSS_BLACK (abyss pixels are black),
+ * GEGL_ABYSS_CLAMP (coordinates are clamped to the abyss rectangle),
+ * GEGL_ABYSS_LOOP (buffer contents are tiled if outside of the abyss rectangle).
+ *
+ * Perform a sampling with the provided @sampler.
+ */
+void              gegl_sampler_get            (GeglSampler    *sampler,
+                                               gdouble         x,
+                                               gdouble         y,
+                                               GeglMatrix2    *scale,
+                                               void           *output,
+                                               GeglAbyssPolicy repeat_mode);
+
+/**
+ * gegl_sampler_get_context_rect:
+ * @sampler: a GeglSampler gotten from gegl_buffer_sampler_new
+ *
+ * Returns:The context rectangle of the given @sampler.
+ */
+const GeglRectangle * gegl_sampler_get_context_rect (GeglSampler *sampler);
+
+/**
+ * gegl_buffer_linear_new: (skip)
  * @extent: dimensions of buffer.
  * @format: desired pixel format.
  *
@@ -380,11 +532,11 @@ GeglInterpolation gegl_buffer_interpolation_from_string (const gchar *string);
  *
  * Returns: a GeglBuffer that can be used as any other GeglBuffer.
  */
-GeglBuffer *gegl_buffer_linear_new           (const GeglRectangle *extent,
-                                              const Babl          *format);
+GeglBuffer *  gegl_buffer_linear_new          (const GeglRectangle *extent,
+                                               const Babl          *format);
 
 /**
- * gegl_buffer_linear_new_from_data:
+ * gegl_buffer_linear_new_from_data: (skip)
  * @data: a pointer to a linear buffer in memory.
  * @format: the format of the data in memory
  * @extent: the dimensions (and upper left coordinates) of linear buffer.
@@ -397,19 +549,19 @@ GeglBuffer *gegl_buffer_linear_new           (const GeglRectangle *extent,
  *
  * Creates a GeglBuffer backed by a linear memory buffer that already exists,
  * of the given @extent in the specified @format. babl_format ("R'G'B'A u8")
- * for instance to make a normal 8bit buffer. 
+ * for instance to make a normal 8bit buffer.
  *
  * Returns: a GeglBuffer that can be used as any other GeglBuffer.
  */
 GeglBuffer * gegl_buffer_linear_new_from_data (const gpointer       data,
-                                                const Babl          *format,
-                                                const GeglRectangle *extent,
-                                                gint                 rowstride,
-                                                GCallback            destroy_fn,
-                                                gpointer             destroy_fn_data);
+                                               const Babl          *format,
+                                               const GeglRectangle *extent,
+                                               gint                 rowstride,
+                                               GDestroyNotify       destroy_fn,
+                                               gpointer             destroy_fn_data);
 
 /**
- * gegl_buffer_linear_open:
+ * gegl_buffer_linear_open: (skip)
  * @buffer: a #GeglBuffer.
  * @extent: region to open, pass NULL for entire buffer.
  * @rowstride: return location for rowstride.
@@ -421,10 +573,10 @@ GeglBuffer * gegl_buffer_linear_new_from_data (const gpointer       data,
  * request is compatible with the underlying data storage direct access
  * to the underlying data is provided.
  */
-gpointer       *gegl_buffer_linear_open      (GeglBuffer          *buffer,
-                                              const GeglRectangle *extent,
-                                              gint                *rowstride,
-                                              const Babl          *format);
+gpointer *      gegl_buffer_linear_open       (GeglBuffer          *buffer,
+                                               const GeglRectangle *extent,
+                                               gint                *rowstride,
+                                               const Babl          *format);
 
 /**
  * gegl_buffer_linear_close:
@@ -436,8 +588,8 @@ gpointer       *gegl_buffer_linear_open      (GeglBuffer          *buffer,
  * it is written back to the buffer. Multiple concurrent users can be handed
  * the same buffer (both raw access and converted).
  */
-void            gegl_buffer_linear_close     (GeglBuffer    *buffer,
-                                              gpointer       linear);
+void            gegl_buffer_linear_close      (GeglBuffer    *buffer,
+                                               gpointer       linear);
 
 
 /**
@@ -447,11 +599,9 @@ void            gegl_buffer_linear_close     (GeglBuffer    *buffer,
  * Return the abyss extent of a buffer, this expands out to the parents extent in
  * subbuffers.
  */
-const GeglRectangle* gegl_buffer_get_abyss  (GeglBuffer           *buffer);
+const GeglRectangle * gegl_buffer_get_abyss   (GeglBuffer           *buffer);
 
 #include <gegl-buffer-iterator.h>
 
-/**
- */
 G_END_DECLS
 #endif
