@@ -28,6 +28,8 @@
 #include <glib-object.h>
 #include <gegl.h>
 
+G_BEGIN_DECLS
+
 /* Extra types needed when coding operations */
 typedef struct _GeglOperation        GeglOperation;
 typedef struct _GeglOperationContext GeglOperationContext;
@@ -71,10 +73,14 @@ GType gegl_module_register_type (GTypeModule     *module,
                                  const GTypeInfo *type_info,
                                  GTypeFlags       flags);
 
+gint            gegl_operation_context_get_level  (GeglOperationContext *self);
 GeglBuffer     *gegl_operation_context_get_source (GeglOperationContext *self,
                                                    const gchar          *padname);
 GeglBuffer     *gegl_operation_context_get_target (GeglOperationContext *self,
                                                    const gchar          *padname);
+void            gegl_operation_context_set_object (GeglOperationContext *context,
+                                                   const gchar          *padname,
+                                                   GObject              *data);
 void            gegl_operation_context_take_object(GeglOperationContext *context,
                                                    const gchar          *padname,
                                                    GObject              *data);
@@ -90,6 +96,49 @@ const gchar   * gegl_extension_handler_get         (const gchar         *extensi
 const gchar   * gegl_extension_handler_get_saver   (const gchar         *extension);
 
 
+/* code template utility, updates the jacobian matrix using
+ * a user defined mapping function for displacement, example
+ * with an identity transform (note that for the identity
+ * transform this is massive computational overhead that can
+ * be skipped by passing NULL to the sampler.
+ *
+ * #define gegl_unmap(x,y,dx,dy) { dx=x; dy=y; }
+ *
+ * gegl_sampler_compute_scale (scale, x, y);
+ * gegl_unmap(x,y,sample_x,sample_y);
+ * gegl_buffer_sample (buffer, sample_x, sample_y, scale, dest, format,
+ *                     GEGL_SAMPLER_LOHALO);
+ *
+ * #undef gegl_unmap      // IMPORTANT undefine map macro
+ */
+#define gegl_sampler_compute_scale(matrix, x, y) \
+{                                       \
+  float ax, ay, bx, by;                 \
+  gegl_unmap(x + 0.5, y, ax, ay);       \
+  gegl_unmap(x - 0.5, y, bx, by);       \
+  matrix.coeff[0][0] = ax - bx;         \
+  matrix.coeff[1][0] = ay - by;         \
+  gegl_unmap(x, y + 0.5, ax, ay);       \
+  gegl_unmap(x, y - 0.5, bx, by);       \
+  matrix.coeff[0][1] = ax - bx;         \
+  matrix.coeff[1][1] = ay - by;         \
+}
+
+typedef struct
+{
+  GObject       parent_instance;
+  void (* get) (GeglSampler    *self,
+                gdouble         x,
+                gdouble         y,
+                GeglMatrix2    *scale,
+                void           *output,
+                GeglAbyssPolicy repeat_mode);
+} SamplerMock;
+
+
+#define gegl_sampler_get(sampler,x,y,scale,dest,repeat) \
+  ((SamplerMock*)(sampler))->get((sampler),(x),(y),(scale),(dest),(repeat))
+
 #include <glib-object.h>
 #include <babl/babl.h>
 #include <operation/gegl-operation.h>
@@ -108,4 +157,5 @@ const gchar   * gegl_extension_handler_get_saver   (const gchar         *extensi
 
 #include <gegl-lookup.h>
 
+G_END_DECLS
 #endif  /* __GEGL_PLUGIN_H__ */
