@@ -86,19 +86,16 @@ G_BEGIN_DECLS
  * initialize everything needed to operate GEGL and parses some
  * standard command line options.  @argc and @argv are adjusted
  * accordingly so your own code will never see those standard
- * arguments. gegl_init() will call g_thread_init(), unless you, or
- * some other code already has initialized gthread.
+ * arguments.
  *
  * Note that there is an alternative way to initialize GEGL: if you
  * are calling g_option_context_parse() with the option group returned
- * by #gegl_get_option_group(), you don't have to call #gegl_init() but
- * you have to call g_thread_init() before any glib or glib dependant code
- * yourself.
+ * by #gegl_get_option_group(), you don't have to call #gegl_init().
  **/
 void           gegl_init                 (gint          *argc,
                                           gchar       ***argv);
 /**
- * gegl_get_option_group:
+ * gegl_get_option_group: (skip)
  *
  * Returns a GOptionGroup for the commandline arguments recognized
  * by GEGL. You should add this group to your GOptionContext
@@ -121,7 +118,7 @@ void           gegl_exit                 (void);
  * Available operations:
  * Gegl provides means to check for available processing operations that
  * can be used with nodes using #gegl_list_operations and for a specified
- * op give a list of properties with #gegl_list_properties.
+ * op give a list of properties with #gegl_operation_list_properties.
  */
 
 /**
@@ -147,14 +144,14 @@ gchar        **gegl_list_operations         (guint *n_operations_p);
 
 
 /**
- * gegl_list_properties:
+ * gegl_operation_list_properties:
  * @operation_type: the name of the operation type we want to query to properties of.
  * @n_properties_p: return location for number of properties.
  *
  * Return value: (transfer container): An allocated array of #GParamSpecs describing the properties
  * of the operation available when a node has operation_type set.
  */
-GParamSpec** gegl_list_properties           (const gchar   *operation_type,
+GParamSpec** gegl_operation_list_properties           (const gchar   *operation_type,
                                              guint         *n_properties_p);
 
 /***
@@ -396,7 +393,7 @@ void          gegl_node_get_valist       (GeglNode      *node,
  */
 
 /**
- * gegl_node_blit:
+ * gegl_node_blit: (skip)
  * @node: a #GeglNode
  * @scale: the scale to render at 1.0 is default, other values changes the
  * width/height of the sampled region.
@@ -544,7 +541,8 @@ GeglNode    * gegl_node_detect           (GeglNode      *node,
  * @node: the node to lookup a paramspec on
  * @property_name: the name of the property to get a paramspec for.
  *
- * Returns the GParamSpec of property or NULL if no such property exists.
+ * Return value: (transfer none): the GParamSpec of property or NULL
+ * if no such property exists.
  */
 GParamSpec  * gegl_node_find_property    (GeglNode      *node,
                                           const gchar   *property_name);
@@ -597,8 +595,8 @@ gint          gegl_node_get_consumers    (GeglNode      *node,
  * Proxies are used to route between nodes of a subgraph contained within
  * a node.
  *
- * Returns an input proxy for the named pad. If no input proxy exists with
- * this name a new one will be created.
+ * Return value: (transfer none): Returns an input proxy for the named pad.
+ * If no input proxy exists with this name a new one will be created.
  */
 GeglNode    * gegl_node_get_input_proxy  (GeglNode      *node,
                                           const gchar   *pad_name);
@@ -607,9 +605,9 @@ GeglNode    * gegl_node_get_input_proxy  (GeglNode      *node,
  * gegl_node_get_operation:
  * @node: a #GeglNode
  *
- * Returns the type of processing operation associated with this node, or
- * NULL if there is no op associated. The special name "GraphNode"
- * is returned if the node is the container of a subgraph.
+ * Return value: The type of processing operation associated with this
+ * node, or NULL if there is no op associated. The special name
+ * "GraphNode" is returned if the node is the container of a subgraph.
  */
 const gchar * gegl_node_get_operation    (const GeglNode *node);
 
@@ -621,8 +619,8 @@ const gchar * gegl_node_get_operation    (const GeglNode *node);
  * Proxies are used to route between nodes of a subgraph contained within
  * a node.
  *
- * Returns a output proxy for the named pad. If no output proxy exists with
- * this name a new one will be created.
+ * Return value: (transfer none):  Returns a output proxy for the named pad.
+ * If no output proxy exists with this name a new one will be created.
  */
 GeglNode    * gegl_node_get_output_proxy (GeglNode      *node,
                                           const gchar   *pad_name);
@@ -804,21 +802,12 @@ void           gegl_processor_set_rectangle (GeglProcessor       *processor,
  *
  * while (gegl_processor_work (processor, &progress))
  *   g_warning ("%f%% complete", progress);
- * gegl_processor_destroy (processor);
+ * g_object_unref (processor);
  */
 gboolean       gegl_processor_work          (GeglProcessor *processor,
                                              gdouble       *progress);
 
 
-/**
- * gegl_processor_destroy:
- * @processor: a #GeglProcessor
- *
- * Frees up resources used by a processing handle.
- */
-void           gegl_processor_destroy       (GeglProcessor *processor);
-
-
 /***
  * GeglConfig:
  *
@@ -834,9 +823,116 @@ void           gegl_processor_destroy       (GeglProcessor *processor);
  * "cache-size" "quality" and "swap", the two first is an integer denoting
  * number of bytes, the secons a double value between 0 and 1 and the last
  * the path of the directory to swap to (or "ram" to not use diskbased swap)
+ *
+ * Return value: (transfer none): a #GeglConfig
  */
 GeglConfig      * gegl_config (void);
 
+/**
+ * gegl_load_module_directory:
+ * @path: the directory to load modules from
+ *
+ * Load all gegl modules found in the given directory.
+ */
+void gegl_load_module_directory (const gchar *path);
+
+/**
+ * gegl_apply_op:
+ * @buffer: the #GeglBuffer to apply onto
+ * @operation_name: name of the operation to apply
+ * @...: the settings for the operation. Zero or more key/value pairs,
+ * ended terminated with NULL.
+ *
+ * Apply the operation to buffer, overwritting the contents of buffer.
+ *
+ */
+void        gegl_apply_op  (GeglBuffer    *buffer,
+                            const gchar   *operation_name,
+                            ...) G_GNUC_NULL_TERMINATED;
+
+/**
+ * gegl_filter_op:
+ * @source_buffer: the source #GeglBuffer for the filter
+ * @operation_name: name of the operation to apply
+ * @...: the settings for the operation. Zero or more key/value pairs,
+ * ended terminated with NULL.
+ *
+ * Apply the operation to source_buffer, returning the result in a new buffer.
+ *
+ * Return value: (transfer full): the result of the filter
+ */
+GeglBuffer *gegl_filter_op (GeglBuffer    *source_buffer,
+                            const gchar   *operation_name,
+                            ...) G_GNUC_NULL_TERMINATED;
+
+
+/**
+ * gegl_render_op:
+ * @source_buffer: the source #GeglBuffer for the filter
+ * @target_buffer: the source #GeglBuffer for the filter
+ * @operation_name: name of the operation to apply
+ * @...: the settings for the operation. Zero or more key/value pairs,
+ * ended terminated with NULL.
+ *
+ * Apply the operation to source_buffer, writing the results to target_buffer.
+ *
+ */
+void        gegl_render_op (GeglBuffer    *source_buffer,
+                            GeglBuffer    *target_buffer,
+                            const gchar   *operation_name,
+                            ...) G_GNUC_NULL_TERMINATED;
+
+/* the following only exist to make gegl_apply and gegl_filter bindable */
+
+/**
+ * gegl_apply_op_valist:
+ * @buffer: the #GeglBuffer to apply onto
+ * @operation_name: name of the operation to apply
+ * @var_args: the settings for the operation. Zero or more key/value pairs,
+ * ended terminated with NULL.
+ *
+ * Apply the operation to buffer, overwritting the contents of buffer.
+ *
+ */
+void        gegl_apply_op_valist (GeglBuffer    *buffer,
+                                  const gchar   *operation_name,
+                                  va_list        var_args);
+
+/**
+ * gegl_filter_op_valist:
+ * @source_buffer: the source #GeglBuffer for the filter
+ * @operation_name: name of the operation to apply
+ * @var_args: the settings for the operation. Zero or more key/value pairs,
+ * ended terminated with NULL.
+ *
+ * Apply the operation to source_buffer, returning the result in a new buffer.
+ *
+ * Return value: (transfer full): the result of the filter
+ */
+GeglBuffer *gegl_filter_op_valist (GeglBuffer   *source_buffer,
+                                   const gchar  *operation_name,
+                                   va_list       var_args);
+
+/**
+ * gegl_render_op_valist:
+ * @source_buffer: the source #GeglBuffer for the filter
+ * @target_buffer: the source #GeglBuffer for the filter
+ * @operation_name: name of the operation to apply
+ * @var_args: the settings for the operation. Zero or more key/value pairs,
+ * ended terminated with NULL.
+ *
+ * Apply the operation to source_buffer, writing the results to target_buffer.
+ *
+ */
+void        gegl_render_op_valist (GeglBuffer   *source_buffer,
+                                   GeglBuffer   *target_buffer,
+                                   const gchar  *operation_name,
+                                   va_list       var_args);
+
+
+
+
+
 
 /**
  * gegl_node: (skip)
@@ -874,6 +970,66 @@ GeglNode *gegl_node (const gchar *op_type,
  */
 GeglNode *gegl_graph (GeglNode *node);
 
+/**
+ * gegl_random_double_range:
+ * @seed: an integer seed, change for different permutation
+ * @x: x coordinate
+ * @y: y coordinate
+ * @z: z coordinate (mipmap level)
+ * @n: number no (each x,y coordinate provides its own sequence of
+ * numbers
+ * @min: minimum value
+ * @max: maxmimum value
+ *
+ * Return a random floating point number in the range specified,
+ * for the given x,y coordinates and seed provided, if multiple different
+ * numbers are needed pass in incrementing n's.
+ */
+double gegl_random_double_range (int seed, int x, int y, int z, int n, double min, double max);
+
+/**
+ * gegl_random_int_range:
+ * @seed: an integer seed, change for different permutation
+ * @x: x coordinate
+ * @y: y coordinate
+ * @z: z coordinate (mipmap level)
+ * @n: number no (each x,y coordinate provides its own sequence of
+ * numbers
+ * @min: minimum value
+ * @max: maxmimum value+1
+ *
+ * Return a random integer point number in the range specified,
+ * for the given x,y coordinates and seed provided, if multiple different
+ * numbers are needed pass in incrementing n's.
+ */
+gint64 gegl_random_int_range    (int seed, int x, int y, int z, int n, int min, int max);
+
+/**
+ * gegl_random_int:
+ * @seed: an integer seed, change for different permutation
+ * @x: x coordinate
+ * @y: y coordinate
+ * @z: z coordinate (mipmap level)
+ * @n: number no (each x,y coordinate provides its own sequence of
+ * numbers
+ *
+ * Return a random integer number in range 0 .. MAX_UINT
+ */
+guint64 gegl_random_int         (int seed, int x, int y, int z, int n);
+
+/**
+ * gegl_random_double:
+ * @seed: an integer seed, change for different permutation
+ * @x: x coordinate
+ * @y: y coordinate
+ * @z: z coordinate (mipmap level)
+ * @n: number no (each x,y coordinate provides its own sequence of
+ * numbers
+ *
+ * Return a random floating point number in range 0.0 .. 1.0.
+ */
+double gegl_random_double       (int seed, int x, int y, int z, int n);
+
 #define GEGL_ALIGNED __restrict__ __attribute__((__aligned__ (16)))
 
 G_END_DECLS
